@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 
 extends qw(Lemonldap::NG::Portal::Auth::_WebForm);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
 # PROPERTIES
 
@@ -19,6 +19,7 @@ has radius => ( is => 'rw' );
 
 has authnLevel => (
     is      => 'rw',
+    lazy    => 1,
     default => sub {
         $_[0]->conf->{radiusAuthnLevel};
     }
@@ -53,7 +54,7 @@ sub authenticate {
         return PE_RADIUSCONNECTFAILED;
     }
 
-    my $res = $self->radius->check_pwd( $req->user, $req->datas->{password} );
+    my $res = $self->radius->check_pwd( $req->user, $req->data->{password} );
     unless ( $res == 1 ) {
         $self->userLogger->warn("Unable to authenticate $req->{user} !");
         $self->setSecurity($req);
@@ -63,7 +64,7 @@ sub authenticate {
 }
 
 sub authLogout {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
