@@ -8,8 +8,9 @@ package Lemonldap::NG::Portal::CDC;
 use strict;
 use Mouse;
 use MIME::Base64;
+use Lemonldap::NG::Common::FormEncode;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Common::PSGI';
 
@@ -29,15 +30,20 @@ sub init {
     my $tmp = Lemonldap::NG::Common::Conf->new( $args->{configStorage} );
     unless ($tmp) {
         $self->error(
-            "Unable to build configuration: $Lemonldap::NG::Common::Conf::msg");
+            "Unable to access configuration: $Lemonldap::NG::Common::Conf::msg"
+        );
         return 0;
     }
-    my $conf = $tmp->getConf();
+    my $lconf = $tmp->getLocalConf('portal') // {};
+    my $conf  = $tmp->getConf();
     unless ( ref($conf) ) {
         $self->error(
             "Unable to load configuration: $Lemonldap::NG::Common::Conf::msg");
         return 0;
     }
+    $lconf->{$_} = $args->{$_}  foreach ( keys %$args );
+    $conf->{$_}  = $lconf->{$_} foreach ( keys %$lconf );
+    $self->SUPER::init($lconf) or return 0;
     $self->cdc_name( $conf->{samlCommonDomainCookieName} || '_saml_idp' );
     $self->cdc_domain( $conf->{samlCommonDomainCookieDomain} );
     $self->logger->debug( "[CDC] Cookie name: " . $self->cdc_name );
@@ -63,7 +69,7 @@ sub handler {
 
     # Request parameter
     my $action = $req->param('action') || "";    # What we do
-    my $idp = $req->param('idp');                # IDP ID in write mode
+    my $idp    = $req->param('idp');             # IDP ID in write mode
 
     # TODO: Control URL
     #my $control_url = $self->_sub('controlUrlOrigin');
@@ -73,9 +79,7 @@ sub handler {
     #}
 
     # Get cookie
-    my %cookies =
-      map { /=/ ? ( split /=/, $_ ) : () } split( /;\s*/, $req->cookie );
-    $cdc_cookie = $cookies{ $self->cdc_name } if %cookies;
+    $cdc_cookie = $req->cookies->{ $self->cdc_name };
 
     if ($cdc_cookie) {
         $self->logger->debug("[CDC] Cookie found with value $cdc_cookie");
@@ -100,7 +104,7 @@ sub handler {
         my $encoded_idp = encode_base64( $idp, '' );
 
         # Remove IDP value if already present
-        $cdc_cookie =~ s/$encoded_idp(\s+)?//g;
+        $cdc_cookie =~ s/$encoded_idp(\s+)?//g if ($cdc_cookie);
 
         # Add a space separator
         $cdc_cookie .= ( $cdc_cookie ? " " : "" );
@@ -110,14 +114,10 @@ sub handler {
             "[CDC] Build cookie $self->{cdc_name} with value $cdc_cookie");
 
         # Build cookie
-        $req->addCookie(
-            $self->p->cookie(
-                name   => $self->cdc_name,
-                value  => $cdc_cookie,
-                domain => $cdc_domain,
-                secure => 1
-            )
-        );
+        push @{ $req->respHeaders },
+            'Set-Cookie' => $self->cdc_name . '='
+          . $cdc_cookie
+          . "; domain=$cdc_domain; secure=1";
     }
 
     # Read request
@@ -152,16 +152,18 @@ sub handler {
         # olStyleUrl can be set to 1 to use & instead of ;
         $urldc .= (
             $cdc_idp
-            ? (
-                $urldc =~ /\?/
-                ? ( $self->{oldStyleUrl} ? '&' : ';' ) . 'idp=' . $cdc_idp
-                : '?idp=' . $cdc_idp
+            ? ( (
+                    $urldc =~ /\?/
+                    ? ( $self->{oldStyleUrl} ? '&' : ';' )
+                    : '?'
+                )
+                . build_urlencoded( idp => $cdc_idp )
               )
             : ''
         );
 
         # Redirect
-        return [ 302, [ Location => $urldc, @{ $req->respHeaders } ], [] ];
+        return [ 302, [ Location => $urldc, $req->spliceHdrs ], [] ];
 
     }
 
@@ -178,7 +180,7 @@ sub handler {
         [
             'Content-Type'   => 'text/plain',
             'Content-Length' => 2,
-            @{ $req->respHeaders }
+            $req->spliceHdrs,
         ],
         ['OK']
     ];
@@ -226,7 +228,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
