@@ -5,8 +5,9 @@ use Mouse;
 use Lemonldap::NG::Common::FormEncode;
 use XML::Simple;
 use Lemonldap::NG::Common::UserAgent;
+use URI;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.12';
 
 # PROPERTIES
 
@@ -26,6 +27,7 @@ has ua => (
 has casSrvList => ( is => 'rw', default => sub { {} }, );
 has casAppList => ( is => 'rw', default => sub { {} }, );
 has spRules    => ( is => 'rw', default => sub { {} }, );
+has spMacros   => ( is => 'rw', default => sub { {} }, );
 
 # RUNNING METHODS
 
@@ -42,7 +44,7 @@ sub loadSrv {
     return 1;
 }
 
-# Load CAS application list, key is the service URL
+# Load CAS application list
 sub loadApp {
     my ($self) = @_;
     unless ( $self->conf->{casAppMetaDataOptions}
@@ -50,22 +52,54 @@ sub loadApp {
     {
         $self->logger->info("No CAS apps found in configuration");
     }
+
     foreach ( keys %{ $self->conf->{casAppMetaDataOptions} } ) {
-        my $tmp =
+
+        my $valid = 1;
+
+        # Load access rule
+        my $rule =
           $self->conf->{casAppMetaDataOptions}->{$_}
-          ->{casAppMetaDataOptionsService};
-        $tmp =~ s#^(https?://[^/]+).*$#$1#;
-        $self->casAppList->{$tmp} = $_;
-        my $rule = $self->conf->{casAppMetaDataOptions}->{$_}
           ->{casAppMetaDataOptionsRule};
         if ( length $rule ) {
             $rule = $self->p->HANDLER->substitute($rule);
             unless ( $rule = $self->p->HANDLER->buildSub($rule) ) {
-                $self->error( 'CAS App rule error: '
+                $self->logger->error(
+                    "Unable to build access rule for CAS Application $_: "
                       . $self->p->HANDLER->tsv->{jail}->error );
-                return 0;
+                $valid = 0;
             }
-            $self->spRules->{$_} = $rule;
+        }
+
+        # Load per-application macros
+        my $macros         = $self->conf->{casAppMetaDataMacros}->{$_};
+        my $compiledMacros = {};
+        for my $macroAttr ( keys %{$macros} ) {
+            my $macroRule = $macros->{$macroAttr};
+            if ( length $macroRule ) {
+                $macroRule = $self->p->HANDLER->substitute($macroRule);
+                if ( $macroRule = $self->p->HANDLER->buildSub($macroRule) ) {
+                    $compiledMacros->{$macroAttr} = $macroRule;
+                }
+                else {
+                    $self->logger->error(
+"Unable to build macro $macroAttr for CAS Application $_: "
+                          . $self->p->HANDLER->tsv->{jail}->error );
+                    $valid = 0;
+                }
+            }
+        }
+
+        if ($valid) {
+            $self->casAppList->{$_} =
+              $self->conf->{casAppMetaDataOptions}->{$_};
+            $self->spRules->{$_}  = $rule;
+            $self->spMacros->{$_} = $compiledMacros;
+        }
+        else {
+            $self->logger->error(
+                "CAS Application $_ has errors and will be ignored");
+
         }
     }
     return 1;
@@ -100,8 +134,7 @@ sub getCasSession {
         );
     }
 
-    my $casSession = Lemonldap::NG::Common::Session->new(
-        {
+    my $casSession = Lemonldap::NG::Common::Session->new( {
             %storage,
             cacheModule        => $self->conf->{localSessionStorage},
             cacheModuleOptions => $self->conf->{localSessionStorageOptions},
@@ -379,7 +412,7 @@ sub validateST {
 
     if ( defined $xml->{'cas:authenticationFailure'} ) {
         $self->logger->error( "Failed to validate Service Ticket $ticket: "
-              . $xml->{'cas:authenticationFailure'} );
+              . $xml->{'cas:authenticationFailure'}->{content} );
         return 0;
     }
 
@@ -425,6 +458,9 @@ sub validateST {
     {
         foreach my $k ( keys %$casAttr ) {
             my $v = $casAttr->{$k};
+            if ( ref($v) eq "ARRAY" ) {
+                $v = join( $self->conf->{multiValuesSeparator}, @$v );
+            }
             utf8::encode($v);
             $k =~ s/^cas://;
             $attrs->{$k} = $v;
@@ -478,6 +514,61 @@ sub retrievePT {
     return $pt;
 }
 
+# Get CAS App from service URL
+sub getCasApp {
+    my ( $self, $uri_param ) = @_;
+
+    my $uri      = URI->new($uri_param);
+    my $hostname = $uri->authority;
+    my $uriCanon = $uri->canonical;
+    return undef unless $hostname;
+
+    my $prefixConfKey;
+    my $longestCandidate = "";
+    my $hostnameConfKey;
+
+    for my $app ( keys %{ $self->casAppList } ) {
+
+        my $candidateUri =
+          URI->new( $self->casAppList->{$app}->{casAppMetaDataOptionsService} );
+        my $candidateHost  = $candidateUri->authority;
+        my $candidateCanon = $candidateUri->canonical;
+
+        # Try to match prefix, remembering the longest match found
+        if ( index( $uriCanon, $candidateCanon ) == 0 ) {
+            if ( length($longestCandidate) < length($candidateCanon) ) {
+                $longestCandidate = $candidateCanon;
+                $prefixConfKey    = $app;
+            }
+        }
+
+        # Try to match host, only if strict matching is disabled
+        unless ( $self->conf->{casStrictMatching} ) {
+            $hostnameConfKey = $app if ( $hostname eq $candidateHost );
+        }
+    }
+
+    # Application found by prefix has priority
+    return $prefixConfKey if $prefixConfKey;
+    $self->logger->warn(
+            "Matched CAS service $hostnameConfKey based on hostname only. "
+          . "This will be deprecated in a future version" )
+      if $hostnameConfKey;
+    return $hostnameConfKey;
+}
+
+# This method returns the host part of the given URL
+# If the URL has no scheme, return it completely
+# http://example.com/uri => example.com
+# foo.bar => foo.bar
+sub _getHostForService {
+    my ( $self, $service ) = @_;
+    return undef unless $service;
+
+    my $uri = URI->new($service);
+    return $uri->scheme ? $uri->host : $uri->as_string;
+}
+
 1;
 __END__
 
