@@ -7,16 +7,12 @@ use Safe;
 extends 'Lemonldap::NG::Portal::Lib::Wrapper';
 with 'Lemonldap::NG::Portal::Lib::OverConf';
 
-our $VERSION = '2.0.0';
-
-has modules => ( is => 'rw', default => sub { {} } );
-
-has rules => ( is => 'rw', default => sub { {} } );
-
-has type => ( is => 'rw' );
-
-has catch => ( is => 'rw', default => sub { {} } );
+our $VERSION = '2.0.12';
 
+has modules    => ( is => 'rw', default => sub { {} } );
+has rules      => ( is => 'rw', default => sub { {} } );
+has type       => ( is => 'rw' );
+has catch      => ( is => 'rw', default => sub { {} } );
 has sessionKey => ( is => 'ro', default => '_choice' );
 
 my $_choiceRules;
@@ -102,7 +98,6 @@ sub checkChoice {
 
     # Check Choice from pdata
     if ( defined $req->pdata->{_choice} ) {
-
         $name = $req->pdata->{_choice};
         $self->logger->debug("Choice $name selected from pdata");
     }
@@ -120,16 +115,30 @@ sub checkChoice {
         }
     }
 
+    unless ($name) {
+
+        # Set by OAuth Resource Owner grant // RESTServer pwdCheck
+        if ( $req->data->{_pwdCheck} and $self->{conf}->{authChoiceAuthBasic} )
+        {
+            $name = $self->{conf}->{authChoiceAuthBasic};
+        }
+    }
+
     unless ($name) {
 
         # Check with other methods
         $name ||=
-             $req->param( $self->conf->{authChoiceParam} )
+             $req->data->{findUserChoice}
+          || $req->param( $self->conf->{authChoiceParam} )
           || $req->userData->{_choice}
           || $req->sessionInfo->{_choice}
           or return 0;
-
-        $self->logger->debug("Choice $name selected");
+        my $from =
+            $req->data->{findUserChoice}                  ? 'findUser'
+          : $req->param( $self->conf->{authChoiceParam} ) ? 'param'
+          : $req->userData->{_choice}                     ? 'userData'
+          :                                                 'sessionInfo';
+        $self->logger->debug("Choice $name selected from $from");
     }
 
     unless ( defined $self->modules->{$name} ) {
@@ -137,13 +146,18 @@ sub checkChoice {
         return 0;
     }
 
-    # Store choice if module loops
-    $req->pdata->{_choice}    = $name;
-    $req->data->{_authChoice} = $name;
+    unless ( $req->data->{findUserChoice} ) {
+
+        # Store choice if module loops
+        $req->pdata->{_choice}       = $name;
+        $req->data->{_authChoice}    = $name;
+        $req->sessionInfo->{_choice} = $name;
+        $self->p->_authentication->authnLevel("${name}AuthnLevel");
+    }
+
     return $name if ( $req->data->{ "enabledMods" . $self->type } );
-    $req->sessionInfo->{_choice} = $name;
-    $req->data->{ "enabledMods" . $self->type } = [ $self->modules->{$name} ];
-    $self->p->_authentication->authnLevel("${name}AuthnLevel");
+    $req->data->{ "enabledMods" . $self->type } =
+      [ $self->modules->{$name} ];
     return $name;
 }
 
@@ -188,7 +202,7 @@ sub _buildAuthLoop {
           split( /[;\|]/, $self->conf->{authChoiceModules}->{$_} );
 
         unless ( $_choiceRules->{$_} ) {
-            $self->logger->error("$_ has no rule !!!");
+            $self->logger->error("$_ has no rule");
             $_choiceRules->{$_} = sub { 1 };
         }
         unless ( $_choiceRules->{$_}->( $req->env ) ) {
@@ -201,7 +215,7 @@ sub _buildAuthLoop {
             if ( $auth and $userDB and $passwordDB ) {
 
                 # Default URL
-                $req->{cspFormAction} ||= '';
+                $req->data->{cspFormAction} ||= {};
                 if (
                     defined $url
                     and not $self->checkXSSAttack( 'URI',
@@ -210,11 +224,9 @@ sub _buildAuthLoop {
                     q%^(https?://)?[^\s/.?#$].[^\s]+$% # URL must be well formatted
                   )
                 {
-                    #$url .= $req->env->{'REQUEST_URI'};
 
-                    # Avoid append same URL
-                    $req->{cspFormAction} .= " $url"
-                      unless $req->{cspFormAction} =~ qr%\b$url\b%;
+                    my $csp_uri = $self->cspGetHost($url);
+                    $req->data->{cspFormAction}->{$csp_uri} = 1;
                 }
                 else {
                     $url .= '#';
@@ -231,8 +243,10 @@ sub _buildAuthLoop {
 
                 # Get displayType for this module
                 no strict 'refs';
-                my $displayType = "Lemonldap::NG::Portal::Auth::${auth}"
-                  ->can('getDisplayType')->( $self, $req );
+                my $displayType = eval {
+                    "Lemonldap::NG::Portal::Auth::${auth}"
+                      ->can('getDisplayType')->( $self, $req );
+                } || 'logo';
 
                 $self->logger->debug(
                     "Display type $displayType for module $auth");
