@@ -8,7 +8,7 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -74,6 +74,9 @@ BEGIN {
 has 'afterSub'  => ( is => 'rw', default => sub { {} } );
 has 'aroundSub' => ( is => 'rw', default => sub { {} } );
 
+# Issuer hooks
+has 'hook' => ( is => 'rw', default => sub { {} } );
+
 has spRules => (
     is      => 'rw',
     default => sub { {} }
@@ -122,6 +125,9 @@ sub init {
 
     # Purge loaded module list
     $self->loadedModules( {} );
+    $self->afterSub(      {} );
+    $self->aroundSub(     {} );
+    $self->hook(          {} );
 
     # Insert `reloadConf` in handler reload stack
     Lemonldap::NG::Handler::Main->onReload( $self, 'reloadConf' );
@@ -182,7 +188,8 @@ sub setPortalRoutes {
       ->addUnauthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
 
       # Logout
-      ->addAuthRoute( logout => 'logout', ['GET'] );
+      ->addAuthRoute( logout => 'logout', ['GET'] )
+      ->addUnauthRoute( logout => 'unauthLogout', ['GET'] );
 
     # Default routes must point to routines declared above
     $self->defaultAuthRoute('');
@@ -204,7 +211,10 @@ sub reloadConf {
     foreach ( qw(_macros _groups), @entryPoints ) {
         $self->{$_} = [];
     }
-    $self->spRules( {} );
+    $self->afterSub(  {} );
+    $self->aroundSub( {} );
+    $self->spRules(   {} );
+    $self->hook(      {} );
 
     # Load conf in portal object
     foreach my $key ( keys %$conf ) {
@@ -239,7 +249,8 @@ sub reloadConf {
 
     # Initialize templateDir
     $self->{templateDir} =
-      $self->conf->{templateDir} . '/' . $self->conf->{portalSkin};
+      $self->conf->{templateDir} . '/' . $self->conf->{portalSkin}
+      if ( $self->conf->{templateDir} and $self->conf->{portalSkin} );
     unless ( -d $self->{templateDir} ) {
         $self->error("Template dir $self->{templateDir} doesn't exist");
         return $self->fail;
@@ -366,9 +377,17 @@ sub reloadConf {
             $p =~ s#https?://([^/]*).*$#$1#;
             $re->add( quotemeta($p) );
         }
+
         foreach my $vhost ( keys %{ $self->conf->{locationRules} } ) {
+            my $expr = quotemeta($vhost);
+
+            # Handle wildcards
+            if ( $vhost =~ /[\%\*]/ ) {
+                $expr =~ s/\\\*/[A-Za-z0-9\.]\*/;
+                $expr =~ s/\\\%/[A-Za-z0-9]\*/;
+            }
+            $re->add($expr);
             $self->logger->debug("Vhost $vhost added in trusted domains");
-            $re->add( quotemeta($vhost) );
             $self->conf->{vhostOptions} ||= {};
             if ( my $tmp =
                 $self->conf->{vhostOptions}->{$vhost}->{vhostAliases} )
@@ -381,7 +400,7 @@ sub reloadConf {
             }
         }
 
-        my $tmp = 'https?://' . $re->as_string . '(?::\d+)?(?:/|$)';
+        my $tmp = '^https?://' . $re->as_string . '(?::\d+)?(?:/|$)';
         $self->trustedDomainsRe(qr/$tmp/);
 
     }
@@ -493,6 +512,25 @@ sub findEP {
             }
         }
     }
+    if ( $obj->can('hook') ) {
+        $self->logger->debug("Found hook in $plugin");
+        my $h = $obj->hook;
+        unless ( ref $h and ref($h) eq 'HASH' ) {
+            $self->logger->error('"hook" endpoint must be a hashref, skipped');
+        }
+        else {
+            foreach my $hookname ( keys %$h ) {
+                my $callback = $h->{$hookname};
+                push @{ $self->hook->{$hookname} }, sub {
+                    eval {
+                        $obj->logger->debug(
+                            "Launching ${plugin}::$callback on hook $hookname");
+                    };
+                    $obj->$callback(@_);
+                };
+            }
+        }
+    }
     $self->logger->debug("Plugin $plugin initializated");
 
     # Rules for menu
@@ -526,7 +564,11 @@ sub loadModule {
         $self->error("Unable to build $module object: $@");
         return 0;
     }
-    ( $obj and $obj->init ) or return 0;
+    unless ( $obj and $obj->init ) {
+        $self->error("$module init failed");
+        return 0;
+    }
+
     $self->loadedModules->{$module} = $obj;
     return $obj;
 }
