@@ -22,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.12';
 
 # PROPERTIES
 
@@ -125,8 +125,8 @@ sub _redirect {
                 ? sub {
 
                     # Restore urldc if auth doesn't need to dial with browser
-                    $self->restoreRequest( $req, $ir );
-                    $self->cleanPdata($req);
+                    $self->restoreRequest( $_[0], $ir );
+                    $self->cleanPdata( $_[0] );
                     return $self->run( @_, @path );
                 }
                 : ()
@@ -212,6 +212,8 @@ sub storeRequest {
     my $info = {};
     $info->{content} = $req->content;
     foreach ( keys %{ $req->env } ) {
+        next if $_ eq "psgi.errors";
+        next if $_ eq "psgi.input";
         $info->{$_} = $req->env->{$_} unless ( ref $req->env->{$_} );
     }
     return $self->_ott->createToken($info);
