@@ -9,13 +9,15 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use URI::Escape;
+use URI;
 use JSON;
+use Lemonldap::NG::Common::Util qw(getPSessionID);
 
 has trOverCache => ( is => 'rw', default => sub { {} } );
 
@@ -89,7 +91,8 @@ sub handler {
     # Save pdata
     if ( $sp or %{ $req->pdata } ) {
         my %v = (
-            name => $self->conf->{cookieName} . 'pdata',
+            name   => $self->conf->{cookieName} . 'pdata',
+            secure => $self->conf->{securedCookie},
             (
                 %{ $req->pdata }
                 ? ( value => uri_escape( JSON::to_json( $req->pdata ) ) )
@@ -131,10 +134,11 @@ sub login {
     return $self->do(
         $req,
         [
-            'controlUrl',        @{ $self->beforeAuth },
-            $self->authProcess,  @{ $self->betweenAuthAndData },
-            $self->sessionData,  @{ $self->afterData },
-            $self->validSession, @{ $self->endAuth },
+            'checkUnauthLogout', 'controlUrl',    # Fix 2342
+            @{ $self->beforeAuth },         $self->authProcess,
+            @{ $self->betweenAuthAndData }, $self->sessionData,
+            @{ $self->afterData },          $self->validSession,
+            @{ $self->endAuth }
         ]
     );
 }
@@ -144,11 +148,11 @@ sub postLogin {
     return $self->do(
         $req,
         [
-            'restoreArgs',                  'controlUrl',
-            @{ $self->beforeAuth },         $self->authProcess,
-            @{ $self->betweenAuthAndData }, $self->sessionData,
-            @{ $self->afterData },          $self->validSession,
-            @{ $self->endAuth },
+            'checkUnauthLogout', 'restoreArgs',            # Fix 2342
+            'controlUrl',        @{ $self->beforeAuth },
+            $self->authProcess,  @{ $self->betweenAuthAndData },
+            $self->sessionData,  @{ $self->afterData },
+            $self->validSession, @{ $self->endAuth }
         ]
     );
 }
@@ -179,8 +183,7 @@ sub postAuthenticatedRequest {
 sub refresh {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
-    my %data          = %{ $req->userData };
-    my $lastAuthLevel = $req->userData->{authenticationLevel};
+    my %data = %{ $req->userData };
     $self->userLogger->notice(
         'Refresh request for ' . $data{ $self->conf->{whatToTrace} } );
     $req->user( $data{_user} || $data{ $self->conf->{whatToTrace} } );
@@ -194,18 +197,12 @@ sub refresh {
     $req->steps( [
             'getUser',
             @{ $self->betweenAuthAndData },
-            'setAuthSessionInfo',
             'setSessionInfo',
-            sub {
-                $req->sessionInfo->{authenticationLevel} = $lastAuthLevel
-                  ;    # Restore previous authentication level (#2179)
-                return PE_OK;
-            },
             $self->groupsAndMacros,
             'setLocalGroups',
             sub {
-                $req->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
-                $req->refresh(1);
+                $_[0]->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
+                $_[0]->refresh(1);
                 return PE_OK;
             },
             'store',
@@ -237,6 +234,24 @@ sub logout {
     );
 }
 
+sub unauthLogout {
+    my ( $self, $req ) = @_;
+    $self->userLogger->info('Unauthenticated logout request');
+    $self->logger->debug('Cleaning pdata');
+    $self->logger->debug("Removing $self->{conf}->{cookieName} cookie");
+    $req->pdata( {} );
+    $req->addCookie(
+        $self->cookie(
+            name    => $self->conf->{cookieName},
+            domain  => $self->conf->{domain},
+            secure  => $self->conf->{securedCookie},
+            expires => 'Wed, 21 Oct 2015 00:00:00 GMT',
+            value   => 0
+        )
+    );
+    return $self->do( $req, [ sub { PE_LOGOUT_OK } ] );
+}
+
 # RUNNING METHODS
 # ---------------
 
@@ -282,7 +297,7 @@ sub do {
                 code    => 401,
                 headers => [
                     'WWW-Authenticate' => "SSO " . $self->conf->{portal},
-                    "Content-Type"     => "application/javascript"
+                    "Content-Type"     => "application/json"
                 ],
             );
         }
@@ -386,7 +401,7 @@ sub autoRedirect {
               )
             {
                 $self->logger->info("Force cleaning pdata");
-                $req->pdata( {} );
+                delete $req->{pdata}->{_url};
             }
             return [ 302, [ Location => $req->{urldc}, $req->spliceHdrs ], [] ];
         }
@@ -471,7 +486,7 @@ sub getPersistentSession {
     return unless ( defined $uid and !$self->conf->{disablePersistentStorage} );
 
     # Compute persistent identifier
-    my $pid = $self->_md5hash($uid);
+    my $pid = getPSessionID($uid);
 
     $info->{_session_uid} = $uid;
 
@@ -536,7 +551,7 @@ sub updatePersistentSession {
 
     if ( $persistentSession->error ) {
         $self->logger->error(
-            "Cannot update persistent session " . $self->_md5hash($uid) );
+            "Cannot update persistent session " . getPSessionID($uid) );
         $self->logger->error( $persistentSession->error );
     }
 }
@@ -567,8 +582,10 @@ sub updateSession {
         foreach ( keys %$infos ) {
             $self->logger->debug("Update sessionInfo $_");
             $self->_dump( $infos->{$_} );
-            $req->{sessionInfo}->{$_} = $self->HANDLER->data->{$_} =
-              $infos->{$_};
+            $req->{sessionInfo}->{$_} = $infos->{$_};
+            if ( $id eq $self->HANDLER->data->{_session_id} ) {
+                $self->HANDLER->data->{$_} = $infos->{$_};
+            }
         }
 
         # Update session in global storage with _updateTime
@@ -630,7 +647,7 @@ sub _deleteSession {
             name    => $self->conf->{cookieName},
             value   => 0,
             domain  => $self->conf->{domain},
-            secure  => 0,
+            secure  => $self->conf->{securedCookie},
             expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
         )
     ) unless ($preserveCookie);
@@ -645,12 +662,6 @@ sub _deleteSession {
     return $session->error ? 0 : 1;
 }
 
-# Return md5(s)
-sub _md5hash {
-    my ( $self, $s ) = @_;
-    return substr( Digest::MD5::md5_hex($s), 0, 32 );
-}
-
 # Check if an URL's domain name is declared in LL::NG config or is declared as
 # trusted domain
 sub isTrustedUrl {
@@ -796,7 +807,7 @@ sub cookie {
     $h{HttpOnly} //= $self->conf->{httpOnly};
     $h{max_age}  //= $self->conf->{cookieExpiration}
       if ( $self->conf->{cookieExpiration} );
-    $h{SameSite} ||= $self->conf->{sameSite};
+    $h{SameSite} ||= $self->cookieSameSite;
 
     foreach (qw(domain path expires max_age HttpOnly SameSite)) {
         my $f = $_;
@@ -877,22 +888,28 @@ sub sendHtml {
     my $csp = $self->csp . "form-action " . $self->conf->{cspFormAction};
     if ( my $url = $req->urldc ) {
         $self->logger->debug("Required urldc : $url");
-        $url =~ s#(https?://[^/]+).*#$1#;
+        $url =~ URIRE;
+        $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
         $self->logger->debug("Set CSP form-action with urldc : $url");
         $csp .= " $url";
     }
     my $url = $args{params}->{URL};
     if ( defined $url ) {
         $self->logger->debug("Required Params URL : $url");
-        if ( $url =~ s#(https?://[^/]+).*#$1# ) {
+        if ( $url =~ URIRE ) {
+            $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
             $self->logger->debug("Set CSP form-action with Params URL : $url");
             $csp .= " $url";
         }
     }
-    if ( defined $req->{cspFormAction} ) {
-        $self->logger->debug(
-            "Set CSP form-action with request URL: " . $req->{cspFormAction} );
-        $csp .= " " . $req->{cspFormAction};
+    if ( defined $req->data->{cspFormAction}
+        and ref( $req->data->{cspFormAction} ) eq "HASH" )
+    {
+        my $request_csp_form_action =
+          join( " ", keys %{ $req->data->{cspFormAction} } );
+        $self->logger->debug( "Set CSP form-action with request URL: "
+              . $request_csp_form_action );
+        $csp .= " " . $request_csp_form_action;
     }
 
     # Set SAML Discovery Protocol in form-action
@@ -922,11 +939,18 @@ sub sendHtml {
     }
 
     # Check if frames need to be embedded
+    # FIXME: we should use $req->data->{cspChildSrc} anywhere an iframe is
+    # created in the code, and remove this
     my @url;
     if ( $req->info ) {
         @url = map { s#https?://([^/]+).*#$1#; $_ }
           ( $req->info =~ /<iframe.*?src="(.*?)"/sg );
     }
+
+    # Update child-src header from request data
+    if ( ref( $req->data->{cspChildSrc} ) eq "HASH" ) {
+        push @url, keys %{ $req->data->{cspChildSrc} };
+    }
     if (@url) {
         $csp .= join( ' ', 'child-src', @url, "'self'" ) . ';';
     }
@@ -1064,7 +1088,7 @@ sub registerLogin {
     }
 
     my $history = $req->sessionInfo->{_loginHistory} ||= {};
-    my $type = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
+    my $type    = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
     $history->{$type} ||= [];
     $self->logger->debug("Current login saved into $type");
 
@@ -1120,6 +1144,17 @@ sub sendJSONresponse {
     my ( $self, $req, $j, %args ) = @_;
     my $res = Lemonldap::NG::Common::PSGI::sendJSONresponse(@_);
 
+    # Handle caching
+    if ( $args{ttl} and $args{ttl} =~ /^\d+$/ ) {
+        push @{ $res->[1] }, 'Cache-Control' => 'public, max-age=' . $args{ttl};
+    }
+    else {
+        push @{ $res->[1] },
+          'Cache-Control' => 'no-cache, no-store, must-revalidate',
+          'Pragma'        => 'no-cache',
+          'Expires'       => '0';
+    }
+
     # If this is a cross-domain request from the portal itself
     # (Ajax SSL to a different VHost)
     # we allow CORS
@@ -1140,6 +1175,18 @@ sub sendJSONresponse {
     return $res;
 }
 
+sub sendRawHtml {
+    my ($self) = $_[0];
+    my $res = Lemonldap::NG::Common::PSGI::sendRawHtml(@_);
+    if ( $self->conf->{corsEnabled} ) {
+        my @cors = split /;/, $self->cors;
+        push @{ $res->[1] }, @cors;
+        $self->logger->debug('Apply following CORS policy :');
+        $self->logger->debug(" $_") for @cors;
+    }
+    return $res;
+}
+
 # Temlate loader
 sub loadTemplate {
     my ( $self, $req, $name, %prm ) = @_;
@@ -1164,4 +1211,16 @@ sub loadTemplate {
     return $tpl->output;
 }
 
+# This method extracts the scheme://host:port part of a URL for use in
+# Content-Security-Polity header
+sub cspGetHost {
+    my ( $self, $url ) = @_;
+    my $uri = $url // "";
+    unless ( $uri->isa("URI") ) {
+        $uri = URI->new($uri);
+    }
+    return (
+        $uri->scheme . "://" . ( $uri->_port ? $uri->host_port : $uri->host ) );
+}
+
 1;
