@@ -62,9 +62,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   portalConsts
   PE_PASSWORD_OK
+  URIRE
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.12';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -78,14 +79,13 @@ has configStorage => (
         $_[0]->{p}->HANDLER->localConfig->{configStorage};
     }
 );
-
 has exportedAttr => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
         my $conf = $_[0]->{conf};
         if ( $conf->{exportedAttr} and $conf->{exportedAttr} !~ /^\s*\+/ ) {
-            return [ split /\s+/, $conf->{exportedAttr} ];
+            return '[' . join( ',', split /\s+/, $conf->{exportedAttr} ) . ']';
         }
         else {
             my @attributes = (
@@ -97,14 +97,17 @@ has exportedAttr => (
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
                 @attributes = ( @attributes, split( /\s+/, $exportedAttr ) );
+
+                # Convert @attributes into hash to remove duplicates
+                my %attributes = map( { $_ => 1 } @attributes );
+                return '[' . join( ',', keys %attributes ) . ']';
             }
 
-            # convert @attributes into hash to remove duplicates
+            # Convert @attributes into hash to remove duplicates
             my %attributes = map( { $_ => 1 } @attributes );
             %attributes =
               ( %attributes, %{ $conf->{exportedVars} }, %{ $conf->{macros} },
               );
-
             return '[' . join( ',', keys %attributes ) . ']';
         }
     }
@@ -245,8 +248,9 @@ sub init {
         mysession => { ':sessionType' => 'updateMySession' },
         ['PUT']
       );
-    extends @parents if ($add);
+    extends @parents               if ($add);
     $self->setTypes( $self->conf ) if ( $self->conf->{restSessionServer} );
+
     return 1;
 }
 
@@ -326,9 +330,8 @@ sub newAuthSession {
     $self->logger->debug(
         "REST authentication result for $req->{user}: code $req->{error}");
 
-    if ( $req->error > 0 ) {
-        return $self->p->sendError( $req, 'Bad credentials', 401 );
-    }
+    return $self->p->sendError( $req, 'Bad credentials', 401 )
+      if ( $req->error > 0 );
     return $self->session( $req, $id );
 }
 
@@ -368,11 +371,13 @@ sub delSession {
     $self->logger->debug("REST request to delete session $id");
     my $res = $self->p->_deleteSession( $req, $session );
     $self->logger->debug(" Result is $res");
+
     return $self->p->sendJSONresponse( $req, { result => $res } );
 }
 
 sub delMySession {
     my ( $self, $req, $id ) = @_;
+
     return $self->delSession( $req, $req->userData->{_session_id} );
 }
 
@@ -380,10 +385,9 @@ sub mysession {
     my ( $self, $req ) = @_;
 
     # 1. whoami
-    if ( defined $req->param('whoami') ) {
-        return $self->p->sendJSONresponse( $req,
-            { result => $req->userData->{ $self->conf->{whatToTrace} } } );
-    }
+    return $self->p->sendJSONresponse( $req,
+        { result => $req->userData->{ $self->conf->{whatToTrace} } } )
+      if defined $req->param('whoami');
 
     if ( defined $req->param('gettoken') ) {
         return $self->p->sendJSONresponse( $req,
@@ -403,7 +407,8 @@ sub mysession {
           if ( $self->p->checkXSSAttack( 'authorizationfor', $req->urldc ) );
 
         # Split URL
-        my ( $host, $uri ) = ( $req->urldc =~ m#^https?://([^/]+)(/.*)?$# );
+        $req->urldc =~ URIRE;
+        my ( $host, $uri ) = ( $3 . ( $4 ? ":$4" : '' ), $5 );
         $uri ||= '/';
         return $self->p->sendError( $req, "Bad URL $req->{urldc}", 400 )
           unless ($host);
@@ -416,6 +421,7 @@ sub mysession {
         $self->logger->debug(" Result is $res");
         return $self->p->sendJSONresponse( $req, { result => $res } );
     }
+
     return $self->p->sendError( $req,
         'whoami or authorizationfor is required', 400 );
 }
@@ -423,7 +429,18 @@ sub mysession {
 sub getMyKey {
     my ( $self, $req, $key ) = @_;
     $key ||= '';
-    $self->logger->debug("Request to get personal session info -> Key : $key");
+    if ($key) {
+        $self->logger->debug(
+            "Request to get personal session info -> Key: $key");
+    }
+    else {
+        my $keys = $self->exportedAttr;
+        $keys =~ s/(?:\[|\])//g;
+        $keys =~ s/,/, /g;
+        $self->logger->debug(
+            "Request to get exported attributes -> Keys: $keys");
+    }
+
     return $self->session(
         $req,
         $req->userData->{_session_id},
@@ -435,6 +452,7 @@ sub updateMySession {
     my ( $self, $req ) = @_;
     my $res   = 0;
     my $mKeys = [];
+
     if ( my $token = $req->param('token') ) {
         if ( $self->ott->getToken($token) ) {
             if ( $req->param('sessionType') eq 'persistent' ) {
@@ -469,9 +487,8 @@ sub updateMySession {
     else {
         $self->logger->error('Update session request without token');
     }
-    unless ($res) {
-        return $self->p->sendError( $req, 'Modification refused', 403 );
-    }
+
+    return $self->p->sendError( $req, 'Modification refused', 403 ) unless $res;
     return $self->p->sendJSONresponse( $req,
         { result => 1, count => $res, modifiedKeys => $mKeys } );
 }
@@ -482,6 +499,7 @@ sub delKeyInMySession {
     my $mKeys = [];
     my $dkey  = $req->param('key');
     my $sub   = $req->param('sub');
+
     if ( my $token = $req->param('token') ) {
         if ( $self->ott->getToken($token) ) {
             if ( $req->param('sessionType') eq 'persistent' ) {
@@ -554,9 +572,8 @@ sub delKeyInMySession {
     else {
         $self->logger->error('Update session request without token');
     }
-    unless ($res) {
-        return $self->p->sendError( $req, 'Modification refused', 403 );
-    }
+
+    return $self->p->sendError( $req, 'Modification refused', 403 ) unless $res;
     return $self->p->sendJSONresponse( $req,
         { result => 1, count => $res, modifiedKeys => $dkey } );
 }
@@ -645,7 +662,7 @@ sub pwdReset {
         return $self->p->sendError( $req, "User not found", 400 );
     }
     $result =
-      $self->p->_passwordDB->modifyPassword( $req, $password, $mail ? 1 : 0 );
+      $self->p->_passwordDB->setNewPassword( $req, $password, $mail ? 1 : 0 );
     $req->{user} = undef;
     $self->conf->{portalRequireOldPassword} = $tmp;
 
@@ -682,7 +699,8 @@ sub pwdConfirm {
     }
 
     $req->user($user);
-    $req->data->{password} = $password;
+    $req->data->{password}  = $password;
+    $req->data->{_pwdCheck} = 1;
 
     if ( $self->p->_userDB ) {
         $req->steps( [ 'getUser', 'authenticate' ] );
@@ -721,6 +739,7 @@ sub getUser {
     }
 
     $req->user( $user || $mail );
+    $req->data->{_pwdCheck} = 1;
 
     # Search user in database
     $req->steps( [
@@ -765,8 +784,8 @@ sub _checkSecret {
             $self->logger->error('Bad key, force denied');
         }
     }
-    return $isValid;
 
+    return $isValid;
 }
 
 1;
