@@ -6,26 +6,29 @@ use Mouse;
 use POSIX qw(strftime);
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_MAILOK
+  PE_NOTOKEN
+  PE_MAILERROR
   PE_BADMAILTOKEN
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_MALFORMEDUSER
-  PE_MAILCONFIRMATION_ALREADY_SENT
-  PE_MAILCONFIRMOK
-  PE_MAILERROR
-  PE_MAILOK
-  PE_NOTOKEN
-  PE_OK
-  PE_REGISTERALREADYEXISTS
-  PE_REGISTERFIRSTACCESS
-  PE_REGISTERFORMEMPTY
   PE_TOKENEXPIRED
+  PE_MAILCONFIRMOK
+  PE_MALFORMEDUSER
+  PE_REGISTERFORMEMPTY
+  PE_REGISTERFIRSTACCESS
+  PE_REGISTERALREADYEXISTS
+  PE_MAILCONFIRMATION_ALREADY_SENT
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.12';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::SMTP', 'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Lib::SMTP
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 # PROPERTIES
 
@@ -111,7 +114,6 @@ sub register {
 # Parameters check
 sub _register {
     my ( $self, $req ) = @_;
-    my %tplPrms;
 
     # Check if it's a first access
     unless ( ( $req->method =~ /^POST$/i and $req->param('mail') )
@@ -298,7 +300,6 @@ sub _register {
           );
 
         # Build mail content
-        $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
         my $tr      = $self->translate($req);
         my $subject = $self->conf->{registerConfirmSubject};
         unless ($subject) {
@@ -309,19 +310,18 @@ sub _register {
         my $html = 1;
 
         # Use HTML template
-        $body = $self->loadTemplate(
+        $body = $self->loadMailTemplate(
             $req,
             'mail_register_confirm',
             filter => $tr,
-            params => \%tplPrms
+            params => {
+                expMailDate => $req->data->{expMailDate},
+                expMailTime => $req->data->{expMailTime},
+                url         => $url,
+                %{ $req->data->{registerInfo} || {} },
+            },
         );
 
-        # Replace variables in body
-        $body =~ s/\$expMailDate/$req->data->{expMailDate}/g;
-        $body =~ s/\$expMailTime/$req->data->{expMailTime}/g;
-        $body =~ s/\$url/$url/g;
-        $body =~ s/\$(\w+)/$req->data->{registerInfo}->{$1}/eg;
-
         # Send mail
         return PE_MAILERROR
           unless $self->send_mail( $req->data->{registerInfo}->{mail},
@@ -358,7 +358,6 @@ sub _register {
     }
 
     # Build mail content
-    $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
     my $tr      = $self->translate($req);
     my $subject = $self->conf->{registerDoneSubject};
     unless ($subject) {
@@ -368,14 +367,6 @@ sub _register {
     my $body;
     my $html = 1;
 
-    # Use HTML template
-    $body = $self->loadTemplate(
-        $req,
-        'mail_register_done',
-        filter => $tr,
-        params => \%tplPrms
-    );
-
     # Build portal url
     my $url = $self->conf->{portal};
     $url =~ s#/*$##;
@@ -387,9 +378,16 @@ sub _register {
         ( $req_url ? ( url => $req_url ) : () ),
       );
 
-    # Replace variables in body
-    $body =~ s/\$url/$url/g;
-    $body =~ s/\$(\w+)/$req->data->{registerInfo}->{$1}/ge;
+    # Use HTML template
+    $body = $self->loadMailTemplate(
+        $req,
+        'mail_register_done',
+        filter => $tr,
+        params => {
+            url => $url,
+            %{ $req->data->{registerInfo} || {} },
+        },
+    );
 
     # Send mail
     return PE_MAILERROR
@@ -455,7 +453,7 @@ sub display {
 
     # Display captcha if it's enabled
     if ( $req->captcha ) {
-        $templateParams{CAPTCHA_SRC} = $req->captcha;
+        $templateParams{CAPTCHA_SRC}  = $req->captcha;
         $templateParams{CAPTCHA_SIZE} = $self->conf->{captcha_size} || 6;
     }
     if ( $req->token ) {
