@@ -309,6 +309,13 @@ m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
               m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
             %fields
         );
+
+        # Add textarea
+        %fields = (
+            $res->[2]->[0] =~
+              m#<textarea.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
+            %fields
+        );
         my $query = buildForm( \%fields );
         foreach my $f (@requiredFields) {
             ok( exists $fields{$f}, qq{ Field "$f" is defined} );
@@ -330,9 +337,7 @@ Verify that result has a C<Lm-Remote-User> header and value is $user
 
 sub expectAuthenticatedAs {
     my ( $res, $user ) = @_;
-    ok( getHeader( $res, 'Lm-Remote-User' ) eq $user,
-        " Authenticated as $user" )
-      or explain( $res->[1], "Lm-Remote-User => $user" );
+    is( getHeader( $res, 'Lm-Remote-User' ), $user, "Authenticated as $user" );
     count(1);
 }
 
@@ -368,6 +373,18 @@ sub expectJSON {
     return $json;
 }
 
+=head4 expectForbidden($res)
+
+Verify that returned code is 403.
+
+=cut
+
+sub expectForbidden {
+    my ($res) = @_;
+    ok( $res->[0] == 403, ' HTTP code is 403' ) or explain( $res->[0], 403 );
+    count(1);
+}
+
 =head4 expectBadRequest($res)
 
 Verify that returned code is 400. Note that it works only for Ajax request
@@ -436,6 +453,23 @@ sub expectCookie {
     return $id;
 }
 
+=head4 expectPdata( $res );
+
+Check if the pdata cookie exists and returns its deserialized value.
+
+=cut
+
+sub expectPdata {
+    my ($res) = @_;
+    my $val = expectCookie( $res, "lemonldappdata" );
+    ok( $val, "Pdata is not empty" );
+    count(1);
+    my $pdata;
+    eval { $pdata = JSON::from_json( uri_unescape($val) ); };
+    diag($@) if $@;
+    return $pdata;
+}
+
 =head4 exceptCspFormOK( $res, $host )
 
 Verify that C<Content-Security-Policy> header allows one to connect to $host.
@@ -464,6 +498,16 @@ sub exceptCspFormOK {
     count(1);
 }
 
+sub expectCspChildOK {
+    my ( $res, $host ) = @_;
+    return 1 unless ($host);
+    my $csp = getHeader( $res, 'Content-Security-Policy' );
+    ok( $csp, "Content-Security-Policy header found" );
+    count(1);
+    like( $csp, qr/child-src[^;]*\Q$host\E/, "Found $host in CSP child-src" );
+    count(1);
+}
+
 =head4 getCookies($res)
 
 Returns an hash ref with names => values of cookies set by server.
@@ -592,6 +636,20 @@ use Mouse;
 
 extends 'Lemonldap::NG::Common::PSGI::Cli::Lib';
 
+# try to find template dir in @INC
+my $templateDir = "site/templates";
+unless ( -d $templateDir ) {
+    for (@INC) {
+        if ( -d "$_/site/templates" ) {
+            $templateDir = "$_/site/templates";
+            last;
+        }
+    }
+    unless ( -d $templateDir ) {
+        die "Could not find template dir";
+    }
+}
+
 our $defaultIni = {
     configStorage => {
         type    => 'File',
@@ -606,7 +664,7 @@ our $defaultIni = {
     logLevel              => 'error',
     cookieName            => 'lemonldap',
     domain                => 'example.com',
-    templateDir           => 'site/templates',
+    templateDir           => $templateDir,
     staticPrefix          => '/static',
     tokenUseGlobalStorage => 0,
     securedCookie         => 0,
@@ -699,16 +757,29 @@ has ini => (
                 uid  => 'french',
                 cn   => 'Frédéric Accents',
                 mail => 'fa@badwolf.org',
-            };
-            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
-                uid  => 'davros',
-                cn   => 'Bad Guy',
-                mail => 'davros@badguy.org',
+                guy  => '',
+                type => '',
             };
             $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{russian} = {
                 uid  => 'russian',
                 cn   => 'Русский',
                 mail => 'ru@badwolf.org',
+                guy  => '',
+                type => '',
+            };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
+                uid  => 'davros',
+                cn   => 'Bad Guy',
+                mail => 'davros@badguy.org',
+                guy  => 'bad',
+                type => 'character',
+            };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{dalek} = {
+                uid  => 'dalek',
+                cn   => 'The Daleks',
+                mail => 'dalek@badguy.org',
+                guy  => 'bad',
+                type => 'mutant',
             };
             push
               @{ $Lemonldap::NG::Portal::UserDB::Demo::demoGroups{earthlings} },
